local E, L, V, P, G = unpack(ElvUI);
local mPlugin = "mMediaTag"
local mMT = E:GetModule(mPlugin);
local DT = E:GetModule("DataTexts");
local addon, ns = ...

--Lua functions
local tinsert = tinsert
local format = format
local strjoin = strjoin
local wipe = wipe

--WoW API / Variables
local _G = _G
local C_ChallengeMode = C_ChallengeMode
local C_MythicPlus = C_MythicPlus
local IsInInstance = IsInInstance
local InCombatLockdown = InCombatLockdown
local IsAddOnLoaded = IsAddOnLoaded

--Variables
local mText = L["Game Menu"]
local menuList = {}
local menuFrame = CreateFrame('Frame', 'mSystemMenu', E.UIParent, 'BackdropTemplate')
menuFrame:SetTemplate("Transparent", true)

local function mColor(color)
	local mSetting = E.db[mPlugin].mMenuColor 
	if mSetting then
		if color == 1 then
			return ns.mColor1
		elseif color == 2 then
			return ns.mColor2
		elseif color == 3 then
			return ns.mColor3
		elseif color == 4 then
			return ns.mColor4
		elseif color == 5 then
			return ns.mColor5
		elseif color == 6 then
			return ns.mColor6
		elseif color == 7 then
			return ns.mColor7
		end
	else
		return ns.mColor1
	end
end


local function OnEvent(self)
	local TextString = mText
	if E.db[mPlugin].SystemMenu.showicon then 
		TextString = format("|T%s:16:16:0:0:128:128|t %s", "Interface\\AddOns\\ElvUI_mMediaTag_Classic\\media\\icons\\system.tga", mText)
	end

	if E.db[mPlugin].InstancInfoName then 
		local inInstance, _ = IsInInstance()
		if inInstance then
			self.text:SetText(mMT:DungeonInfoName())
		else
			self.text:SetFormattedText(mMT:mClassColorString(), TextString)
		end
	else
		self.text:SetFormattedText(mMT:mClassColorString(), TextString)
	end
end

local function OnClick(self, button)
	DT.tooltip:Hide()
	if button == "LeftButton" then
		wipe(menuList)--menuList = {}
		menuList = {
			{lefttext = format("%s%s|r", mColor(2), CHARACTER_BUTTON), isTitle = false,
			func = function() ToggleCharacter("PaperDollFrame") end},
			{lefttext = format("%s%s|r", mColor(2), SPELLBOOK_ABILITIES_BUTTON), isTitle = false,
			func = function() if not SpellBookFrame:IsShown() then ShowUIPanel(SpellBookFrame) else HideUIPanel(SpellBookFrame) end end},
			{lefttext = format("%s%s|r", mColor(2), TALENTS_BUTTON), isTitle = false,
				func = function()
					if not PlayerTalentFrame then
						TalentFrame_LoadUI()
					end
					
					if not PlayerTalentFrame:IsShown() then
						ShowUIPanel(PlayerTalentFrame)
					else
						HideUIPanel(PlayerTalentFrame)
					end
			end},
			{lefttext = "", isTitle = true, func = function() end},

			{lefttext = format("%s%s|r", mColor(4), "ElvUI"), isTitle = false,
				func = function() 
					if InCombatLockdown() then _G.UIErrorsFrame:AddMessage(E.InfoColor.._G.ERR_NOT_IN_COMBAT) return end
					E:ToggleOptionsUI()
			end},
			
			{lefttext = "", isTitle = true, func = function() end},
			
			{lefttext = format("%s%s|r", mColor(1), MAINMENU_BUTTON), isTitle = false,
				func = function()
					if ( not GameMenuFrame:IsShown() ) then
						if ( VideoOptionsFrame:IsShown() ) then
							VideoOptionsFrameCancel:Click();
						elseif ( AudioOptionsFrame:IsShown() ) then
							AudioOptionsFrameCancel:Click();
						elseif ( InterfaceOptionsFrame:IsShown() ) then
							InterfaceOptionsFrameCancel:Click();
						end
						CloseMenus();
						CloseAllWindows()
						ShowUIPanel(GameMenuFrame);
					else
						HideUIPanel(GameMenuFrame);
						MainMenuMicroButton_SetNormal();
					end
			end},
			
			{lefttext = "", isTitle = true, func = function() end},
			
			{lefttext = format("%s%s|r", mColor(5), HELP_BUTTON), isTitle = false, func = function() ToggleHelpFrame() end}
		}
		mMT:mDropDown(menuList, menuFrame, self, 160, 2)
	end
end

local function OnEnter(self)
	local nhc, hc, myth, mythp, other, titel, tip = mMT:mColorDatatext()
	
	DT.tooltip:AddLine(mText)
	if E.db[mPlugin].InstancInfoToolTip then
		local mInctanceInfo = mMT:InctanceInfo()
		if mInctanceInfo then
			DT.tooltip:AddLine(" ")
			DT.tooltip:AddLine(mInctanceInfo[1] or "-")
			DT.tooltip:AddLine(mInctanceInfo[2] or "-")
			DT.tooltip:AddLine(mInctanceInfo[3] or "-")
		end
		
		local inInstance, _ = IsInInstance()
		if inInstance then
			local infoinInstance = mMT:DungeonInfo()
			DT.tooltip:AddLine(" ")
			DT.tooltip:AddLine(infoinInstance[1] or "-")
			DT.tooltip:AddLine(infoinInstance[2] or "-")
			DT.tooltip:AddLine(infoinInstance[3] or "-")
		end
	end
	
	DT.tooltip:AddDoubleLine(ns.mName, format("%sVer.|r %s%s|r" , titel, other, ns.mVersion))
	DT.tooltip:AddLine(" ")
	DT.tooltip:AddLine(format("%s %s%s|r", ns.LeftButtonIcon, tip, L["Click left to open the main menu."]))
	DT.tooltip:AddLine(format("%s %s%s|r", ns.RightButtonIcon, tip, L["Left click to open the menu."]))
	DT.tooltip:Show()
end

local function OnLeave(self)
	DT.tooltip:Hide()
end

DT:RegisterDatatext("mGameMenu", nil, {"CHALLENGE_MODE_START", "CHALLENGE_MODE_COMPLETED", "PLAYER_ENTERING_WORLD", "UPDATE_INSTANCE_INFO"}, OnEvent, nil, OnClick, OnEnter, OnLeave, mText, nil, nil)